import datetime
import json
import logging
import urllib.parse
import uuid

import requests
from dateutil import parser
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import JsonResponse

# from django.contrib.auth.mixins import LoginRequiredMixin
from django.shortcuts import redirect, render
from django.utils import timezone
from django.utils.decorators import method_decorator
from django.views import View
from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework_simplejwt.authentication import JWTAuthentication

from .models import StravaToken
from dj_rest_auth.registration.views import SocialLoginView
from allauth.socialaccount.providers.apple.views import AppleOAuth2Adapter

logger = logging.getLogger('kudosdeluxe')

ACTIVITY_COLORS = {
    'Run': '#FF6384',
    'Ride': '#36A2EB',
    'Swim': '#FFCE56',
    'Hike': '#4BC0C0',
    # Add more activity types and colors as needed
}

URI_STRAVA_LINK = "kudosdeluxe:link"


class AppleLoginView(SocialLoginView):
    adapter_class = AppleOAuth2Adapter


class GetStateView(LoginRequiredMixin, View):
    """
    Generates a unique state and associates it with the authenticated user's StravaToken.
    """
    def get(self, request):
        state = str(uuid.uuid4())
        try:
            strava_token = StravaToken.objects.get(user=request.user)
            strava_token.oauth_state = state
            strava_token.save()
            return JsonResponse({'state': state})
        except StravaToken.DoesNotExist:
            return JsonResponse({'error': 'StravaToken not found.'}, status=404)


class StravaOAuthCallbackView(View):
    def get(self, request):
        logger.debug("Received Strava OAuth callback.")
        code = request.GET.get('code')
        state = request.GET.get('state')
        error = request.GET.get('error')

        if error:
            logger.error(f"Strava OAuth Error: {error}")
            return render(request, 'strava_callback.html', {'success': False, 'error': error})

        if not code or not state:
            logger.error("Authorization code or state not found in callback.")
            return render(request, 'strava_callback.html', {'success': False, 'error': "Authorization code or state not found."})

        try:
            # Retrieve StravaToken using the state
            strava_token = StravaToken.objects.get(oauth_state=state)
        except StravaToken.DoesNotExist:
            logger.error(f"Invalid state parameter: {state}")
            return render(request, 'strava_callback.html', {'success': False, 'error': "Invalid state parameter."})

        # Prepare the payload for token exchange
        token_url = "https://www.strava.com/oauth/token"
        payload = {
            "client_id": settings.STRAVA_CLIENT_ID,
            "client_secret": settings.STRAVA_CLIENT_SECRET,
            "code": code,
            "grant_type": "authorization_code"
        }

        try:
            # Exchange the authorization code for tokens
            response = requests.post(token_url, data=payload)
            response.raise_for_status()
            tokens = response.json()

            # Extract tokens and athlete information
            access_token = tokens.get('access_token')
            refresh_token = tokens.get('refresh_token')
            expires_at = tokens.get('expires_at')
            athlete = tokens.get('athlete')

            if not all([access_token, refresh_token, expires_at, athlete]):
                logger.error("Incomplete token information received from Strava.")
                return render(request, 'strava_callback.html', {'success': False, 'error': "Incomplete token information."})

            # Update StravaToken
            strava_token.strava_access_token = access_token
            strava_token.strava_refresh_token = refresh_token
            strava_token.strava_expires_at = expires_at
            strava_token.strava_athlete_id = athlete.get('id')
            strava_token.oauth_state = uuid.uuid4()  # Invalidate the used state
            strava_token.save()

            logger.debug("Successfully exchanged code for tokens.")
            return render(request, 'strava_callback.html', {'success': True, 'athlete': athlete})

        except requests.exceptions.HTTPError as http_err:
            logger.error(f"HTTP error during token exchange: {http_err}")
            return render(request, 'strava_callback.html', {'success': False, 'error': "Failed to exchange authorization code for tokens."})
        except requests.exceptions.RequestException as req_err:
            logger.error(f"Request exception during token exchange: {req_err}")
            return render(request, 'strava_callback.html', {'success': False, 'error': "Error communicating with Strava."})
        except Exception as e:
            logger.error(f"Unexpected error during token exchange: {e}")
            return render(request, 'strava_callback.html', {'success': False, 'error': "An unexpected error occurred."})


class GetStravaTokenView(View):
    """
    Receives the authorization code and state from the iOS app, associates them, and prepares for the callback.
    """

    def post(self, request):
        try:
            data = json.loads(request.body)
            code = data.get('code')
            state = data.get('state')
            
            if not code or not state:
                return JsonResponse({'error': 'Code or state missing.'}, status=400)
            
            # Find the StravaToken by state
            try:
                strava_token = StravaToken.objects.get(oauth_state=state)
            except StravaToken.DoesNotExist:
                return JsonResponse({'error': 'Invalid state parameter.'}, status=400)
            
            # The StravaOAuthCallbackView will handle the exchange using the state
            # You might need to trigger the OAuth callback process here or ensure the callback URL includes the state
            
            return JsonResponse({'message': 'State and code received.'}, status=200)
        
        except json.JSONDecodeError:
            return JsonResponse({'error': 'Invalid JSON.'}, status=400)
        

def get_activity_color(activity_type):
    return ACTIVITY_COLORS.get(activity_type, '#CCCCCC')

@login_required
def strava_login(request):
    """
    Initiates the Strava OAuth flow by redirecting the user to Strava's authorization page.
    """
    client_id = settings.STRAVA_CLIENT_ID
    redirect_uri = settings.STRAVA_REDIRECT_URI
    response_type = 'code'
    scope = 'read,activity:read_all'  # Adjust scopes as needed
    approval_prompt = 'auto'  # Use 'force' to force approval each time

    # Construct the authorization URL
    auth_url = 'https://www.strava.com/oauth/authorize'
    params = {
        'client_id': client_id,
        'redirect_uri': redirect_uri,
        'response_type': response_type,
        'scope': scope,
        'approval_prompt': approval_prompt,
    }
    url = f"{auth_url}?{urllib.parse.urlencode(params)}"

    return redirect(url)


def kudos_deluxe_view(request):
    token = None
    if request.user.is_authenticated:
        tokens = StravaToken.objects.filter(user=request.user)
        if tokens.exists():
            token = tokens.first()
    return render(request, "page_kudos_deluxe.html", context={'token': token})


def kudos_deluxe_link_view(request):
    token = None
    if request.user.is_authenticated:
        token = StravaToken.objects.filter(user=request.user).first()
    return render(request, "page_kudos_deluxe_link.html", context={'token':token})


@login_required
def strava_callback(request):
    """
    Handles the OAuth callback from Strava, exchanges the authorization code for tokens,
    and stores them in the database.
    """
    code = request.GET.get('code')
    error = request.GET.get('error')

    if error:
        messages.error(request, f"Strava authorization failed: {error}")
        return redirect(URI_STRAVA_LINK)

    if not code:
        messages.error(request, "No authorization code provided.")
        return redirect('kudosdeluxe')

    # Exchange authorization code for access token
    token_url = 'https://www.strava.com/oauth/token'
    payload = {
        'client_id': settings.STRAVA_CLIENT_ID,
        'client_secret': settings.STRAVA_CLIENT_SECRET,
        'code': code,
        'grant_type': 'authorization_code',
    }

    response = requests.post(token_url, data=payload)
    if response.status_code != 200:
        messages.error(request, "Failed to obtain access token from Strava.")
        return redirect(URI_STRAVA_LINK)

    token_data = response.json()
    access_token = token_data['access_token']
    refresh_token = token_data['refresh_token']
    expires_at = token_data['expires_at']  # UNIX timestamp

    # Convert UNIX timestamp to datetime
    expires_datetime = datetime.datetime.fromtimestamp(expires_at, tz=datetime.timezone.utc).timestamp()

    # Save or update tokens in the database
    StravaToken.objects.update_or_create(
        user=request.user,
        defaults={
            'strava_access_token': access_token,
            'strava_refresh_token': refresh_token,
            'strava_expires_at': expires_datetime,
        }
    )

    messages.success(request, "Strava account linked successfully.")
    return redirect(URI_STRAVA_LINK)



@login_required
def strava_activities(request):
    """
    Fetches and displays the user's Strava activities for the current week starting from Monday.
    Includes data preparation for a stacked bar chart and a doughnut chart visualization.
    """
    try:
        token = StravaToken.objects.get(user=request.user)
    except StravaToken.DoesNotExist:
        messages.error(request, "Strava account not linked.")
        return redirect(URI_STRAVA_LINK)

    # Refresh token if expired
    if token.is_expired():
        refresh_url = 'https://www.strava.com/oauth/token'
        payload = {
            'client_id': settings.STRAVA_CLIENT_ID,
            'client_secret': settings.STRAVA_CLIENT_SECRET,
            'grant_type': 'refresh_token',
            'refresh_token': token.refresh_token,
        }

        response = requests.post(refresh_url, data=payload)
        if response.status_code != 200:
            messages.error(request, "Failed to refresh access token.")
            return redirect(URI_STRAVA_LINK)

        token_data = response.json()
        token.strava_access_token = token_data['access_token']
        token.strava_refresh_token = token_data['refresh_token']
        token.strava_expires_at = datetime.datetime.fromtimestamp(token_data['expires_at'], tz=datetime.timezone.utc).timestamp()
        token.save()

    # Calculate Monday of the current week
    today = timezone.now().date()
    start_of_week = today - datetime.timedelta(days=today.weekday())  # Monday
    start_of_week_datetime = datetime.datetime.combine(start_of_week, datetime.time.min, tzinfo=datetime.timezone.utc)
    start_of_week_timestamp = int(start_of_week_datetime.timestamp())

    headers = {'Authorization': f'Bearer {token.strava_access_token}'}
    activities_url = 'https://www.strava.com/api/v3/athlete/activities'

    params = {
        'per_page': 30,  # Adjust as needed
        'page': 1,
        'after': start_of_week_timestamp,
    }

    response = requests.get(activities_url, headers=headers, params=params)
    if response.status_code != 200:
        messages.error(request, "Failed to fetch activities from Strava.")
        return redirect(URI_STRAVA_LINK)

    activities = response.json()

    # Prepare data for the bar chart
    days = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']
    day_minutes = {day: {} for day in days}  # { 'Monday': {'Run': 30, 'Ride': 60}, ... }
    total_minutes = 0  # Initialize total minutes
    activity_type_totals = {}  # For doughnut chart

    for activity in activities:
        try:
            start_date_local = parser.isoparse(activity.get('start_date_local'))
            day = start_date_local.strftime('%A')  # Full weekday name
        except (ValueError, TypeError):
            continue  # Skip activities with invalid dates

        activity_type = activity.get('type', 'Other')
        duration_min = activity.get('moving_time', 0) / 60  # seconds to minutes
        total_minutes += duration_min

        # Aggregate for bar chart
        if activity_type not in day_minutes[day]:
            day_minutes[day][activity_type] = 0
        day_minutes[day][activity_type] += duration_min

        # Aggregate for doughnut chart
        if activity_type not in activity_type_totals:
            activity_type_totals[activity_type] = 0
        activity_type_totals[activity_type] += duration_min

    # Prepare datasets for the bar chart
    bar_datasets = []
    activity_types = sorted(activity_type_totals.keys())  # Consistent order

    for activity_type in activity_types:
        data = []
        for day in days:
            data.append(round(day_minutes[day].get(activity_type, 0), 2))
        bar_datasets.append({
            'label': activity_type,
            'data': data,
            'backgroundColor': get_activity_color(activity_type),
        })

    # Prepare data for the doughnut chart
    doughnut_labels = list(activity_type_totals.keys())
    doughnut_data = [round(activity_type_totals[atype], 2) for atype in doughnut_labels]
    doughnut_colors = [get_activity_color(atype) for atype in doughnut_labels]

    context = {
        'activities': activities,
        'days': days,
        'bar_datasets': bar_datasets,
        'doughnut_labels': doughnut_labels,
        'doughnut_data': doughnut_data,
        'doughnut_colors': doughnut_colors,
        'total_minutes': round(total_minutes, 2),
    }

    return render(request, 'page_kudos_deluxe_show.html', context)


# class StravaGetTokenView(APIView):
#     """
#     Endpoint to retrieve Strava tokens for authenticated users.
#     """
#     authentication_classes = [JWTAuthentication]
#     permission_classes = [IsAuthenticated]

#     def post(self, request):
#         # Implement logic to retrieve tokens associated with the authenticated user
#         # For demonstration, assuming tokens are stored in the user model
#         user = request.user
#         tokens = {
#             "access_token": user.strava_access_token,
#             "refresh_token": user.strava_refresh_token
#         }
#         return Response(tokens, status=status.HTTP_200_OK)


class StravaRefreshTokenView(APIView):
    """
    Endpoint to refresh Strava access tokens using the refresh token.
    """
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        refresh_token = request.data.get('refresh_token')
        if not refresh_token:
            return Response({"error": "Refresh token not provided."}, status=status.HTTP_400_BAD_REQUEST)

        token_url = "https://www.strava.com/oauth/token"
        client_id = settings.STRAVA_CLIENT_ID
        client_secret = settings.STRAVA_CLIENT_SECRET

        payload = {
            "client_id": client_id,
            "client_secret": client_secret,
            "refresh_token": refresh_token,
            "grant_type": "refresh_token"
        }

        try:
            response = requests.post(token_url, data=payload)
            response.raise_for_status()
            new_tokens = response.json()

            # Update tokens in the user model
            user = request.user
            user.strava_access_token = new_tokens.get('access_token')
            user.strava_refresh_token = new_tokens.get('refresh_token')
            user.save()

            return Response(new_tokens, status=status.HTTP_200_OK)
        except requests.exceptions.HTTPError as err:
            print(f"HTTPError during token refresh: {err}")
            return Response({"error": "Failed to refresh token."}, status=response.status_code)
        except Exception as e:
            print(f"Unexpected error during token refresh: {e}")
            return Response({"error": "An unexpected error occurred."}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



# class StravaOAuthCallbackView(View):
#     """
#     Handles Strava OAuth2 callback, exchanges code for tokens, and renders a confirmation page with JS redirection.
#     """

#     def get(self, request):
#         logger.debug("Received Strava OAuth callback.")
#         # Extract the authorization code from the query parameters
#         code = request.GET.get('code')
#         error = request.GET.get('error')

#         if error:
#             messages.error(request, f"Strava OAuth Error: {error}")
#             logger.error(f"Strava OAuth Error: {error}")
#             return render(request, 'strava_callback.html', {'success': False, 'error': error})

#         if not code:
#             messages.error(request, "No authorization code provided in the callback.")
#             logger.error("No authorization code provided in the callback.")
#             return render(request, 'strava_callback.html', {'success': False, 'error': "No authorization code provided."})

#         # Prepare the payload for token exchange
#         token_url = "https://www.strava.com/oauth/token"
#         client_id = settings.STRAVA_CLIENT_ID
#         client_secret = settings.STRAVA_CLIENT_SECRET
#         redirect_uri = "https://geder.at/strava/callback/"  # Must match the registered redirect_uri

#         payload = {
#             "client_id": client_id,
#             "client_secret": client_secret,
#             "code": code,
#             "grant_type": "authorization_code"
#         }

#         try:
#             # Exchange the authorization code for access and refresh tokens
#             response = requests.post(token_url, data=payload)
#             response.raise_for_status()  # Raise an exception for HTTP errors
#             tokens = response.json()

#             # Extract tokens and athlete information
#             access_token = tokens.get('access_token')
#             refresh_token = tokens.get('refresh_token')
#             expires_at = tokens.get('expires_at')
#             athlete = tokens.get('athlete')

#             if not all([access_token, refresh_token, expires_at, athlete]):
#                 messages.error(request, "Incomplete token information received from Strava.")
#                 logger.error("Incomplete token information received from Strava.")
#                 return render(request, 'strava_callback.html', {'success': False, 'error': "Incomplete token information."})

#             # Associate tokens with the authenticated user
#             # Since the view is now accessible without login, you need another way to associate tokens with a user.
#             # One approach is to use the 'state' parameter to identify the user.
#             # For simplicity, assume you have a mechanism to identify the user based on the 'state' parameter.

#             # Example using 'state' parameter
#             state = request.GET.get('state')
#             if not state:
#                 messages.error(request, "Missing state parameter.")
#                 logger.error("Missing state parameter.")
#                 return render(request, 'strava_callback.html', {'success': False, 'error': "Missing state parameter."})

#             # Lookup the user based on the state
#             # Implement this according to your application's logic
#             user = self.get_user_from_state(state)
#             if not user:
#                 messages.error(request, "Invalid state parameter.")
#                 logger.error("Invalid state parameter.")
#                 return render(request, 'strava_callback.html', {'success': False, 'error': "Invalid state parameter."})

#             try:
#                 # Assuming you have a Profile model linked to the User model
#                 user.stravatoken.strava_access_token = access_token
#                 user.stravatoken.strava_refresh_token = refresh_token
#                 user.stravatoken.strava_expires_at = expires_at
#                 user.stravatoken.strava_athlete_id = athlete.get('id')
#                 user.stravatoken.save()
#             except AttributeError:
#                 messages.error(request, "User profile is not properly configured.")
#                 logger.error("User profile is not properly configured.")
#                 return render(request, 'strava_callback.html', {'success': False, 'error': "User profile configuration error."})

#             # Render the callback template with success status
#             return render(request, 'strava_callback.html', {'success': True, 'athlete': athlete})

#         except requests.exceptions.HTTPError as http_err:
#             error_message = f"HTTP error occurred during token exchange: {http_err}"
#             print(error_message)
#             logger.error(error_message)
#             messages.error(request, error_message)
#             return render(request, 'strava_callback.html', {'success': False, 'error': "Failed to exchange authorization code for tokens."})
#         except requests.exceptions.RequestException as req_err:
#             error_message = f"Request exception occurred: {req_err}"
#             print(error_message)
#             logger.error(error_message)
#             messages.error(request, error_message)
#             return render(request, 'strava_callback.html', {'success': False, 'error': "An error occurred while communicating with Strava."})
#         except Exception as e:
#             error_message = f"Unexpected error: {e}"
#             print(error_message)
#             logger.error(error_message)
#             messages.error(request, error_message)
#             return render(request, 'strava_callback.html', {'success': False, 'error': "An unexpected error occurred."})
    
    def get_user_from_state(self, state: str):
        """
        Retrieve the user associated with the given state.
        Implement this method based on your application's logic.
        For example, you might have a model that maps state to user.
        """
        # Example implementation:
        # try:
        #     oauth_state = OAuthState.objects.get(state=state)
        #     return oauth_state.user
        # except OAuthState.DoesNotExist:
        #     return None
        
        # Placeholder implementation:
        return None


class GetStravaTokensView(APIView):
    """
    Endpoint for the iOS app to retrieve Strava access and refresh tokens.
    """
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user
        tokens = {
            "access_token": user.stravatoken.strava_access_token,
            "refresh_token": user.stravatoken.strava_refresh_token,
            "expires_at": user.stravatoken.strava_expires_at,
            "athlete_id": user.stravatoken.strava_athlete_id
        }

        if not tokens["access_token"]:
            return Response({"error": "Strava account not connected."}, status=status.HTTP_400_BAD_REQUEST)

        return Response(tokens, status=status.HTTP_200_OK)